import {IEditSession, createEditSession} from 'brace';

/* tslint:disable */
/**
 * stores the session of the editors
 * use this class as external storage because we cannot save the session in redux properly...
 * we need to store the sessions in case we got a login timeout and the user is logged off and then logins in again
 * and want to continue working... at the time the login is displayed the editor is unmounted
 *
 * it's often better to remove all editors when loading the page (e.g. do exercise site)
 * and then after getting the user solution to manually create the AceEditorsStorageHelper editor tabs (in componentDidMount)
 * (e.g. the simple file editor do not automatically add AceEditorsStorageHelper tabs fir them we normally do it in the reducer e.g.
 * @see getEditorExerciseReducer)
 * the tabs are automatically created on multi file editor mount but only for the visible editor (e.g. after solution has no tabs then because is not visible)
 * for an example to do it manually you can
 * @see _loadTutorViewData
 *
 * files are added/removed via
 * @see addTabToEditor
 * @see removeTabToEditor
 * and called e.g. in the multi file editor when we add/remove a file
 *
 * so e.g. the {@link SimpleAceEditor} tries to find an old session with the same tab id
 *   so make sure when you want to show the same content e.g. after and modal was shown that the same id is used!!
 *   you can see {@link FeedbackEditorView} how this can be used when we have no real id (e.g. no backend file id)
 *
 */
export class AceEditorsStorageHelper {
  private constructor() {
  }

  static editorsMap: EditorMap = {}

  public static addEditor(id: string, initialTab: AceStoredTabLike) {
    const initialSession = createEditSession(initialTab.content, `ace/mode/${initialTab.language}` as any)

    const initialWrappedTab: EditorWrapperTab = {
      session: initialSession,
      tab: initialTab
    }
    this.editorsMap[id] = [initialWrappedTab]
  }

  public static addEditorWithTabs(id: string, initialTabs: AceStoredTabLike[]) {

    const tabs: EditorWrapperTab[] = []
    for (let i = 0; i < initialTabs.length; i++) {
      const initialTab = initialTabs[i]

      const initialSession = createEditSession(initialTab.content, `ace/mode/${initialTab.language}` as any)

      const initialWrappedTab: EditorWrapperTab = {
        session: initialSession,
        tab: initialTab
      }

      tabs.push(initialWrappedTab)

    }
    this.editorsMap[id] = tabs
  }

  public static addTabToEditor(id: string, tab: AceStoredTabLike) {

    const tabs = this.editorsMap[id]

    if (!tabs) return

    const initialSession = createEditSession(tab.content, `ace/mode/${tab.language}` as any)

    const initialWrappedTab: EditorWrapperTab = {
      session: initialSession,
      tab: tab
    }

    tabs.push(initialWrappedTab)
  }

  /**
   *
   * @param id
   * @param tab
   * @returns true: id was removed, false: tab was not found
   */
  public static removeTabToEditor(id: string, tab: AceStoredTabLike): boolean {

    const tabs = this.editorsMap[id]

    if (!tabs) return false

    const index = tabs.findIndex(p => p.tab.id === tab.id)

    if (index === -1) return false

    tabs.splice(index,1)

    return true
  }

  public static getEditor(id: string): EditorWrapperTab[] | undefined {
    return this.editorsMap[id]
  }

  /**
   *
   * @param content
   * @param language teh language without ace/mode/ only the lang
   */
  public static createNewSession(content: string, language: string) {
    return createEditSession(content, `ace/mode/${language}` as any)
  }

  /**
   * call this when the dialog is dismissed and we want to reset the session/editor
   * @param id
   */
  public static removeEditor(id: string): void {

    if (!this.editorsMap[id]) return

    delete this.editorsMap[id]
  }

  /**
   * call this when the site is changed
   */
  public static removeAll() {
    this.editorsMap = {}
  }

  public static exportAll(): EditorMapExport {

    const copy: EditorMapExport = {}

    for(const key in this.editorsMap) {

      const entry = this.editorsMap[key]

      copy[key] = entry.map<EditorWrapperTabExport>(p => {
        return {
          tab: {
            ...p.tab
          },
          session: {
            // @ts-ignore
            modeId: p.session.getMode().$id,
            value:p.session.getValue()
          }
        }
      })

    }

    return copy
  }

}


interface EditorMap {
  [id: string]: EditorWrapperTab[]
}


export interface EditorWrapperTab {

  /**
   * this is the tab that should refer to the session
   * it is also used to create the initial session but then get's out of sync because not automatically updated
   * the editors should take care of the sync
   */
  tab: AceStoredTabLike
  /**
   * this is a reference to the real session which is always up to date
   */
  session: IEditSession
}

/**
 * the storage only keeps tack of some editor tab props...
 * there is no need to e.g. keep track of the file name... the content is only used so we can use editor blur
 * and the id to find the real redux file
 */
export interface AceStoredTabLike {
  /**
   * the id of the original file
   */
  readonly id: number
  /**
   * the content
   */
  readonly content: string

  /**
   * the programming language
   * this can not be changed...
   */
  readonly language: string
}

export interface EditorMapExport {
  [id: string]: EditorWrapperTabExport[]
}


export interface EditorWrapperTabExport {

  /**
   * this is the tab that should refer to the session
   * it is also used to create the initial session but then get's out of sync because not automatically updated
   * the editors should take care of the sync
   */
  tab: AceStoredTabLike
  /**
   * this is a reference to the real session which is always up to date
   */
  session: {
    modeId: string //e.g. ace/mode/java
    value: string //editor content
  }
}
